#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 08/30/2001
#################################################################################
#
# Realm Configuration - Script part of Webchat designed for access control lists
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Audit;						# Audit trail
use Ethereal::Database;						# Database handler
use Ethereal::Date;						# Date handler
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table handle


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my @allow;							# Allow list
my @block;							# Block list

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Multiple handle handling
	@allow = $cgi->param('ALLOW');
	@block = $cgi->param('BLOCK');

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login hanle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationAdmin())
	{
		# Title
		print $cgi->h1('Realm Access Administration'), "\n";

		# Logic Tree
		# Access Control
		if (defined($param->{'ROOM'}))
		{ AccessControl($database, $cgi, $param, \@allow, \@block); }

		# Realm Listing
		else
		{ RealmSelect($database, $cgi, $param); }

	}

	# Document footer
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Realm Select
#
# Basic display and prompt user for a selection on which realm to adminster and
# make modifications of some type to.

sub RealmSelect
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash

	my $table    = new Ethereal::Table($cgi);		# Table handler

	my @realms;						# Array of realms
	my $realms;						# Popup menu

	
	#####################
	# Program area

	# Prepare of drop down
	$database->GetListNonPrivate(\@realms);
	$realms = $cgi->popup_menu('ROOM', \@realms);

	# HTML Display
	# Star form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";

	# Start table
	$table->MakeTop();

	# Realm selection
	$table->MakeValid('<B>Select Realm</B>', $realms);
	$table->MakeBlank();

	# Submit and bottom
	$table->MakeSingle($cgi->submit('Continue'));
	$table->MakeBottom();

	# End of form and embed
	print $cgi->end_form(), "\n"; 
}


#####################
# Access Control
#
# Access Control Lists is essentially the role of this script.  This specific 
# subroutine is aimed speficially at handling who can access which Realm
# and perform supervisory functions.

sub AccessControl
{
	#####################
	# Data Members

	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $allow    = shift;					# Access granted list
	my $block    = shift;					# Access denied list
	
	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $audit;						# Audit handle
	my $handle;						# Handle name

	my $badd     = '>>';					# Addition button
	my $brem     = '<<';					# Remove button

	my $inline;						# Inline parameter list
	my $url      = $cgi->url();				# Self referencing link

	my @allow;						# Allowed
	my @block;						# Blocked


	#####################
	# Program Files

	# Audit handle
	$audit = new Ethereal::Audit($database, $cgi, $param);

	# Display realm
	print $cgi->h2($param->{'ROOM'});

	# Data handling 
	# Additions
	if (defined($param->{$badd}))
	{
		foreach $handle (@{$block})
		{
			# Insert Entry
			$data->do("INSERT INTO RealmSupervisor 
				(RealmName,
				 PuppeteerLogin)
				VALUES (?,?)", {},

			 $param->{'ROOM'},
			 $handle);

			# Audit table
			$audit->AccessAdd($param->{'ROOM'}, $handle);	
		}

		# Cleanup	
		$param->Cleanup($badd);
	}

	# Removal
	elsif (defined($param->{$brem}))
	{
		foreach $handle (@{$allow})
		{
			# Remove data
			$data->do("DELETE FROM RealmSupervisor
				WHERE PuppeteerLogin=?
				  AND RealmName=?", {},

				$handle,
				$param->{'ROOM'});

			# Audit table
			$audit->AccessAdd($param->{'ROOM'}, $handle);	
		}

		# Cleanup
		$param->Cleanup($brem);
	}
	


	# Data retreival
	# Retrieve variables
	$database->GetList(\@block, "SELECT PuppeteerLogin 
		FROM  Puppeteer 
		WHERE PuppeteerSuperPrivs='yes' 
		ORDER BY PuppeteerLogin"); 
	$database->GetList(\@allow, "SELECT p.PuppeteerLogin 
		FROM  Puppeteer p, RealmSupervisor r 
		WHERE p.PuppeteerLogin = r.PuppeteerLogin
		  AND r.RealmName=?
		  AND p.PuppeteerSuperPrivs='yes'
		ORDER BY p.PuppeteerLogin", $param->{'ROOM'});

	# Generate inline
	($inline) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}); 


	# Form display
	# Form start
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";

	# Display
	print "<CENTER><TABLE WIDTH=\"500\">\n",
	      "\t<TR>\n\t\t<TD COLSPAN=\"3\"><CENTER><B><A HREF=\"$url$inline\">[Back]</A></B></CENTER></TD>\n\t</TR>\n",
	      "\t<TR>\n",
	      "\t\t<TD><B>Supervisors</B></TD>",
	      "\t\t<TD WIDTH=\"50\">&nbsp;</TD>",
	      "\t\t<TD><B>Realm Supervisors</B></TD>",
	      "\t</TR>\n",
	      "\t<TR>\n",
	      "\t\t<TD>", $cgi->scrolling_list('BLOCK', \@block, $block[0], '5', 'true'), "</TD>",
	      "\t\t<TD WIDTH=\"50\">", $cgi->submit($badd), "<BR><BR>", $cgi->submit($brem), "</TD>",
	      "\t\t<TD>", $cgi->scrolling_list('ALLOW', \@allow, $allow[0], '5', 'true'), "</TD>",
	      "\t</TR>\n",
	      "</TABLE></CENTER>";

	# Form end
	print $cgi->end_form(), "\n";
}
