#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 01/22/2002
#################################################################################
#
# Dice - Script part of Webchat designed to send dice rolls
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Comm;						# Communication handler
use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Option;						# Option handling
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table handler


#################################################################################
# Data Members
#################################################################################
my $comm;							# Communications panel handle
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my %realm;							# Realm hash

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);	

	# Fetch hash
	$database->GetHashRealm(\%realm);

	# Login and Comm handle
	$login    = new Ethereal::Login($database, $cgi, $param); 
	$comm     = new Ethereal::Comm($database, $cgi);

	# Authenthication
	if ($login->GetVerificationRealm())
	{
		# Prevent bans from use
		$login->CheckLegality(\%realm);

		# Make sure there is a color
		unless (defined($param->{'COLOR'}))
		{
			# Determine and assign
			my ($tsize, $tcolor, $tface) = $database->DataGetFont($param->{'CHAR'});
			$param->{'COLOR'} = $tcolor;
			$param->{'FACE'}  = $tface;
		}

		# Enforce size
		$param->{'SIZE'} = 3;


		# Only do the following if posting is needed
		if (defined($param->{'ROLL'}))
		{
			RollHandler($param, \%realm);
			$comm->DataWrite($param);
		}

		# Display panel
		DiceDisplay($database, $cgi, $param, \%realm);
	}


#################################################################################
# Sub-Routines
#################################################################################

####################
# Dice Display
#
# Gives the users options for the roll of dice.  This means the selection of the
# type of die as well as the number of specific rolls.

sub DiceDisplay							# Database, Cgi, Param
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Cgi handle
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Chat options

	my $option   = new Ethereal::Option($cgi);		# Option handler

	my $type;						# Selection type
	my $freq;						# Frequency
	my $panel;						# Dice roller panel

	my $dtype = $param->{'DTYPE'};				# Dice type
	my $dfreq = $param->{'DFREQ'};				# Dice frequency

	my @types;						# Dice types


	####################
	# Program area

	# Find types
	$option->Split(\@types, $realm->{'OptDiceTypes'});

	# Assign values
	$type = (defined($dtype)) ? $dtype : $types[0];
	$freq = (defined($dfreq)) ? $dfreq : 1;
	
        # Retreive dice roller panel
        $panel = $database->FetchRealmDice($param->{'ROOM'});

        # Widgets
        # Assigning values
        my $wdiceroll = $cgi->submit($realm->{'TxtRoll'});
        my $wdicefreq = $cgi->textfield('DFREQ', $freq, 3, 2);
        my $wdicetype = $cgi->popup_menu('DTYPE',  \@types,  $type); 

	# Form values
	my $wformstrt = $cgi->start_form();
	my $wformhidn = $param->EmbedNormal($cgi, %{$param}, 'ROLL', 'True');
	my $wformends = $cgi->end_form();


	# Font selection
	$panel =~ s/WDICEROLL/$wdiceroll/gs;
	$panel =~ s/WDICEFREQ/$wdicefreq/gs;
	$panel =~ s/WDICETYPE/$wdicetype/gs;

	# Form
	$panel =~ s/WFORMSTRT/$wformstrt$wformhidn/gs;
	$panel =~ s/WFORMENDS/$wformends/gs;



	# Display HTML
	# Document header
	print $cgi->header;

	# Document
	print "$panel";
}


####################
# Roll Handler
#
# Will perform a roll of the dice as specified by the user himself.   This is done
# using a pseudo random seed.

sub RollHandler							# Database, Cgi, Param
{
	####################
	# Data members
	my $param = shift;					# Parameter handle
	my $realm = shift;					# Chat options

	my $results;						# Formatted results
	my $total;						# Active total

	my $count = 0;						# Counter
	my $rand;						# Random number


	####################
	# Program area

	# Adjust seed
	srand();

	# Make sure numbers are received
	$param->{'DFREQ'} = ($param->{'DFREQ'} =~ /^\d+$/) ? $param->{'DFREQ'} : 1;

	# Enforce an proper amount of posts:w
	$param->{'DFREQ'} = ($param->{'DFREQ'} < 250) ? $param->{'DFREQ'} : 250;

	# Contruct begin results
	$rand = int((rand() * $param->{'DTYPE'}) + 1);
	$results = "$realm->{TagDiceRoll} (<B>$param->{DFREQ}d$param->{DTYPE}</B>) $rand";
	$total += $rand;
	$count++;

	# Rolls for each roll
	for ($count; $count < $param->{'DFREQ'}; $count++)
	{
		# Generate number
		$rand = int((rand() * $param->{'DTYPE'}) + 1);

		# Continue contruct
		$results = "$results + $rand";

		# Work on total
		$total += $rand;
	} 
	
	# Finish contruct
	$results = "$results = $total";

	# Assign contruct as post
	$param->{'POST'} = $results;
	$param->{'TARGET'} = 'dice roll';
}
